"""A simple timer context manager and decorator."""

import functools
import time
from collections.abc import Callable, Iterator
from contextlib import contextmanager
from typing import ParamSpec, TypeVar

P = ParamSpec("P")
R = TypeVar("R")


@contextmanager
def timer(decimal_places: int = 5) -> Iterator[None]:
    """Time the indented block and print time elapsed."""
    start_time = time.perf_counter()
    yield None
    elapsed = time.perf_counter() - start_time
    print(f"{elapsed:.{decimal_places}f} seconds elapsed.")


def timed(decimal_places: int = 5) -> Callable[[Callable[P, R]], Callable[P, R]]:
    """Print seconds elapsed in decorated function after call."""

    def _timed(func: Callable[P, R]) -> Callable[P, R]:
        @functools.wraps(func)
        def _wrapper(*args: P.args, **kwargs: P.kwargs) -> R:
            start_time = time.perf_counter()
            result = func(*args, **kwargs)
            elapsed = time.perf_counter() - start_time
            print(f"{elapsed:.{decimal_places}f} seconds elapsed.")
            return result

        return _wrapper

    return _timed



"""Some tools."""

from importlib.metadata import PackageNotFoundError, version

__version__: str | None

try:
    __version__ = version("sammiesutils")
except PackageNotFoundError:
    __version__ = None



"""Typed wrappers for functools.cache and functools.lru_cache decorators."""

import functools
from collections.abc import Callable
from typing import ParamSpec, TypeVar

P = ParamSpec("P")
R = TypeVar("R")


def cache(func: Callable[P, R]) -> Callable[P, R]:
    """Preserve function signature and wrap with functools.cache decorator."""
    @functools.wraps(func)
    @functools.cache
    def _wrapper(*args: P.args, **kwargs: P.kwargs) -> R:
        return func(*args, **kwargs)

    return _wrapper  # type: ignore


def lru_cache(
    func: Callable[P, R], maxsize: int | None = 128, typed: bool = False
) -> Callable[P, R]:
    """Preserve function signature and wrap with functools.lru_cache decorator."""
    @functools.wraps(func)
    @functools.lru_cache(maxsize=maxsize, typed=typed)
    def _wrapper(*args: P.args, **kwargs: P.kwargs) -> R:
        return func(*args, **kwargs)

    return _wrapper  # type: ignore
